package rip.iwakura.civilcore;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.bukkit.Bukkit;

import rip.iwakura.civilcore.types.DbPlayer;
import rip.iwakura.civilcore.types.Team;

public class Database {
  public ArrayList<DbPlayer> players = new ArrayList<DbPlayer>();
  public ArrayList<Team> teams = new ArrayList<Team>();

  private Connection c = null;

  public boolean tableExists(String table) throws SQLException {
    if (c.getMetaData().getTables(null, null, table, null).next()) {
      return true;
    }
    return false;
  }

  public DbPlayer getPlayerByName(String name) {
    List<DbPlayer> filtered = players.stream().filter(
        s -> s.name.equals(name))
        .collect(Collectors.toList());
    if (filtered.isEmpty()) return null;
    return filtered.getFirst();
  }

  public Team getTeamByName(String name) {
    List<Team> filtered = teams.stream().filter(
        s -> s.name.equals(name))
        .collect(Collectors.toList());
    if (filtered.isEmpty()) return null;
    return filtered.getFirst();
  }

  public void update(String sql) throws SQLException {
    Statement stmt = c.createStatement();
    stmt.executeUpdate(sql);
    stmt.close();
  }
  
  public void addPlayerToTeam(Team team, DbPlayer player) throws SQLException {
    String sql = "UPDATE players SET TEAM_ID = ? WHERE name = ?; ";
    PreparedStatement stmt = c.prepareStatement(sql);

    stmt.setInt(1, team.id);
    stmt.setString(2, player.name);

    stmt.executeUpdate();

    stmt.close();

    refreshPlayers();
  }

  public void refreshTeams() throws SQLException {
    teams.clear();
    Statement stmt = c.createStatement();
    ResultSet rs = stmt.executeQuery("SELECT * FROM teams;");

    while (rs.next()) {
      String name = rs.getString("name");
      String prefix = rs.getString("prefix");
      int id = rs.getInt("team_id"); 

      Team team = new Team(name,prefix,id);
      teams.add(team);
    }
    stmt.close();
    rs.close();
  }

  public List<DbPlayer> getPlayersInTeam(String team_name) {
    List<DbPlayer> filtered = players.stream().filter(
        s -> {
          if (s.team.name == null) return false;
          return s.team.name.equals(team_name);
        })
      .collect(Collectors.toList());
    if(filtered.isEmpty()) return null;
    return filtered;
  }

  public void refreshPlayers() throws SQLException {
    players.clear();
    String sql = "SELECT players.NAME AS player_name, teams.NAME AS team_name, teams.TEAM_ID as t_team_id, teams.PREFIX AS team_prefix " +
                 "FROM players LEFT JOIN teams ON players.TEAM_ID = teams.TEAM_ID";

    Statement stmt = c.createStatement();
    ResultSet rs = stmt.executeQuery(sql);


    while (rs.next()) { 
      String p_name = rs.getString("player_name");
      String t_name = rs.getString("team_name");
      int t_id = rs.getInt("t_team_id");
      String t_prefix = rs.getString("team_prefix");

      Team team = new Team(t_name, t_prefix, t_id);
      DbPlayer player = new DbPlayer(p_name, team);
      players.add(player);
    }
    stmt.close();
    rs.close();
  }

  public void insertPlayer(DbPlayer player) throws SQLException {
    String sql = "INSERT INTO players (NAME,TEAM_ID) " +
                 "VALUES(?, ?);";
    PreparedStatement stmt = c.prepareStatement(sql);

    stmt.setString(1, player.name);
    if(player.team != null) {
      stmt.setInt(2, player.team.id);
    } else {
      stmt.setNull(2, Types.NULL);
    }

    stmt.executeUpdate();

    stmt.close();

    refreshPlayers();
  }

  public void insertTeam(Team team) throws SQLException {
    String sql = "INSERT INTO teams (NAME,PREFIX) " +
                 "VALUES(?, ?);";
    PreparedStatement stmt = c.prepareStatement(sql);

    stmt.setString(1, team.name);
    stmt.setString(2, team.prefix);

    stmt.executeUpdate();

    stmt.close();

    refreshTeams();
  }



  public void connect(String url) throws SQLException {
    try {
      Class.forName("org.postgresql.Driver");
      c = DriverManager.getConnection(url);
    } catch (Exception e) {
      e.printStackTrace();
      Bukkit.getLogger().info("Failed to connect to the database.");
      Bukkit.getServer().getPluginManager().disablePlugin(new CivilCore());
      return;
    }
    Bukkit.getLogger().info("Connected to the database.");
    if (!tableExists("teams")) {
      String sql = "CREATE TABLE teams " +
                   "(TEAM_ID SERIAL PRIMARY KEY," +
                   " NAME           TEXT NOT NULL," +
                   " PREFIX         TEXT NOT NULL)";
      update(sql);
    }
    if (!tableExists("players")) {
      String sql = "CREATE TABLE players " +
                   "(NAME TEXT PRIMARY KEY NOT NULL," +
                   " TEAM_ID INT, " +
                   " CONSTRAINT TEAM" +
                   "     FOREIGN KEY(TEAM_ID)" +
                   "     REFERENCES teams(TEAM_ID))";
      update(sql);
    }

    refreshTeams();
    refreshPlayers();

    Bukkit.getLogger().info("Database has been initialized.");
  }
}
