package rip.iwakura.civilcore.commands;

import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import net.md_5.bungee.api.ChatColor;
import org.jetbrains.annotations.NotNull;
import rip.iwakura.civilcore.Utils;
import rip.iwakura.civilcore.CivilCore;
import rip.iwakura.civilcore.types.DbPlayer;

import java.util.List;
import java.util.Objects;

public class TeamChatCommand implements CommandExecutor {
  private final CivilCore civilCore;

  public TeamChatCommand(CivilCore civilCore) {
    this.civilCore = civilCore;
  }

  private String getPrefix(String name, boolean inTeamChat) {
    DbPlayer dbPlayer = civilCore.db.getPlayerByName(name);
    return (inTeamChat ? ChatColor.DARK_GREEN + "TEAM >" + ChatColor.RESET : "")
            + (dbPlayer.team.prefix == null ?
            "" :
            ChatColor.translateAlternateColorCodes('&', inTeamChat ? "" : "[" + dbPlayer.team.prefix + "]"));
  }

  @Override
  public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, String[] args) {
    args = Objects.requireNonNull(Utils.parser(args)).toArray(new String[0]);

    if (!(sender instanceof Player p)) return true;
    if (args.length == 0) {
      if (civilCore.teamChatRegister.contains(p)) {
        civilCore.teamChatRegister.remove(p);
        p.sendMessage(ChatColor.GREEN + "You are now in the " + ChatColor.GOLD + "GLOBAL" + ChatColor.GREEN + " channel");
      } else {
        civilCore.teamChatRegister.add(p);
        p.sendMessage(ChatColor.GREEN + "You are now in the " + ChatColor.GOLD + "TEAM" + ChatColor.GREEN + " channel");
      }
      return true;
    }

    String message = String.join(" ", args);

    String player_name = p.getName();
    boolean inTeamChat = civilCore.teamChatRegister.contains(p);

    DbPlayer dbPlayer = civilCore.db.getPlayerByName(player_name);

    if (inTeamChat) {
      // Send message to global chat
      civilCore.teamChatRegister.remove(p);
      String formatted = String.format("%s%s: %s", getPrefix(player_name, false) + " ", player_name, message);
      Bukkit.broadcastMessage(formatted);
      civilCore.bot.sendMessage(p, message);
      civilCore.teamChatRegister.add(p);
    } else {
      // Send message to team chat
      civilCore.teamChatRegister.add(p);
      String formatted = String.format("%s%s: %s", getPrefix(player_name, true) + " ", player_name, message);
      List<DbPlayer> players = civilCore.db.getPlayersInTeam(dbPlayer.team.name);

      if (players == null) return true;

      for (DbPlayer team_dbp : players) {
        Player team_p = Bukkit.getPlayer(team_dbp.name);

        if (team_p == null) continue;

        team_p.sendMessage(formatted);
      }
      civilCore.teamChatRegister.remove(p);
    }

    return true;
  }
}