package rip.iwakura.civilcore.commands;

import java.sql.SQLException;

import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;

import net.md_5.bungee.api.ChatColor;
import rip.iwakura.civilcore.Utils;
import rip.iwakura.civilcore.CivilCore;
import rip.iwakura.civilcore.types.DbPlayer;
import rip.iwakura.civilcore.types.Team;

public class TeamCommand implements CommandExecutor {
  private CivilCore civilCore = null;

  public TeamCommand(CivilCore civilCore) {
    this.civilCore = civilCore;
  }

  @Override
  public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
    args = Utils.parser(args).toArray(new String[0]);

    if(args.length == 0) {
      sender.sendMessage(ChatColor.RED + "Usage: /team <create | destroy | add | remove | list> [options]");
      return true;
    };

    switch (args[0]) {
      case "list":
        if (args.length < 2) {
          sender.sendMessage(ChatColor.RED + "Usage: /team list <teams | players>");
          return true;
        }

        switch (args[1]) {
          case "players":
            for (DbPlayer dbPlayer : civilCore.db.players) { 
              if (dbPlayer.team != null) sender.sendMessage(ChatColor.GREEN + String.format("%s belonging to team %s", dbPlayer.name, dbPlayer.team.name));
              if (dbPlayer.team == null) sender.sendMessage(ChatColor.GREEN + String.format("%s does not belong to any team.", dbPlayer.name));
            }
            return true;
          case "teams":
            for (Team team : civilCore.db.teams) {
              sender.sendMessage(ChatColor.GREEN + String.format("%s with prefix %s", team.name, ChatColor.translateAlternateColorCodes('&', "&r" + team.prefix)));
            }
            return true;
          default:
            sender.sendMessage(ChatColor.RED + "Usage: /team list <teams | players>");
            return true;
        }
      case "create":
        if(args.length < 3) {
          sender.sendMessage(ChatColor.RED + "Usage: /team create <name> <prefix>");
          return true;
        };

        Team team = new Team(args[1], args[2], null);

			  try {
				  civilCore.db.insertTeam(team);
          sender.sendMessage(ChatColor.GREEN + String.format("Successfully made team %s with prefix %s", args[1], ChatColor.translateAlternateColorCodes('&', "&r" + args[2])));
			  } catch (Exception e) {
				  e.printStackTrace();
          Utils.exceptionHandler(e, sender);
			  }

        break;
      case "add":
        if(args.length < 3) {
          sender.sendMessage(ChatColor.RED + "Usage: /team add <player> <team name>");
          return true;
        };

        Team found_team = civilCore.db.getTeamByName(args[2]);
        DbPlayer dbPlayer = civilCore.db.getPlayerByName(args[1]);

        if (found_team == null) {
          sender.sendMessage(ChatColor.RED + "The team you are referencing does not exist.");
          return true;
        }

        String success_msg = ChatColor.GREEN + String.format("Successfully added %s to team %s.", args[1], args[2]);

        if(dbPlayer == null) {
          dbPlayer = new DbPlayer(args[1], found_team);
          try {
			      civilCore.db.insertPlayer(dbPlayer);
            sender.sendMessage(success_msg);
		      } catch (Exception e) {
			      e.printStackTrace();
            Utils.exceptionHandler(e, sender);
		      }
          return true;
        }

			  try {
				  civilCore.db.addPlayerToTeam(found_team, dbPlayer);
          sender.sendMessage(success_msg);
			  } catch (Exception e) {
				  e.printStackTrace();
          Utils.exceptionHandler(e, sender);
			  }
        break;
    }
    return true;
  }
}
