package rip.iwakura.civilcore.discord;

import java.util.EnumSet;

import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.player.AsyncPlayerChatEvent;
import org.checkerframework.common.reflection.qual.GetConstructor;

import club.minnced.discord.webhook.WebhookClient;
import club.minnced.discord.webhook.WebhookClientBuilder;
import club.minnced.discord.webhook.send.WebhookMessage;
import club.minnced.discord.webhook.send.WebhookMessageBuilder;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import net.dv8tion.jda.api.requests.GatewayIntent;
import rip.iwakura.civilcore.CivilCore;
import net.dv8tion.jda.api.events.session.ReadyEvent;
import rip.iwakura.civilcore.types.DbPlayer;

public class Discord extends ListenerAdapter {
  private String token;
  private CivilCore civilCore;

  public String channelId;

  private TextChannel channel;
  private WebhookClient webhook;

  public JDA bot;

  private EnumSet<GatewayIntent> intents = EnumSet.of(GatewayIntent.GUILD_MESSAGES, GatewayIntent.MESSAGE_CONTENT, GatewayIntent.GUILD_WEBHOOKS);
  
  public Discord(String token, String channelId, CivilCore civilCore){
    this.civilCore = civilCore;
    this.channelId = channelId;
    this.token = token;
  }

  public void initialize() {
    this.bot = JDABuilder.createLight(token, intents)
      .addEventListeners(new MessageReceive(civilCore))
      .addEventListeners(this)
      .build();
  }

  public void sendMessage(AsyncPlayerChatEvent e) {
    DbPlayer dbPlayer = civilCore.db.getPlayerByName(e.getPlayer().getName());
    String prefix = (dbPlayer.team.prefix == null ?
            "" :
            "[" + dbPlayer.team.prefix + "]");

    String player_name = e.getPlayer().getName();
    WebhookMessageBuilder builder = new WebhookMessageBuilder()
      .setUsername(prefix + " " + player_name)
      .setAvatarUrl(String.format("https://mc-heads.net/avatar/%s", player_name))
      .setContent(e.getMessage());
    webhook.send(builder.build());
    //channel.sendMessage(String.format("%s: %s", e.getPlayer().getName(), e.getMessage())).complete();
  }

  public void sendMessage(Player p, String message) {
    DbPlayer dbPlayer = civilCore.db.getPlayerByName(p.getName());
    String prefix = (dbPlayer.team.prefix == null ?
      "" :
      "[" + dbPlayer.team.prefix + "]");

    WebhookMessageBuilder builder = new WebhookMessageBuilder()
      .setUsername(prefix + " " + p.getName())
      .setAvatarUrl(String.format("https://mc-heads.net/avatar/%s", p.getName()))
      .setContent(message);
    webhook.send(builder.build());
    //channel.sendMessage(String.format("%s: %s", p.getName(), message)).complete();
  }

  public void sendMessage(String message, String name) {
    DbPlayer dbPlayer = civilCore.db.getPlayerByName(name);
    String prefix = (dbPlayer.team.prefix == null ?
            "" :
            "[" + dbPlayer.team.prefix + "]");

    WebhookMessageBuilder builder = new WebhookMessageBuilder()
      .setUsername(prefix + " " + name)
      .setAvatarUrl(String.format("https://mc-heads.net/avatar/%s", name))
      .setContent(message);
    webhook.send(builder.build());
    //channel.sendMessage(String.format("%s: %s", name, message)).complete();
  }

  @Override
  public void onReady(ReadyEvent e) {
	  this.channel = bot.getTextChannelById(channelId);
    String webhook_token = civilCore.getConfig().getString("discord.webhook.token");
    if (webhook_token != null) {
      Long webhook_id = Long.parseLong(civilCore.getConfig().getString("discord.webhook.id"));
      this.webhook = new WebhookClientBuilder(webhook_id, webhook_token).build();
    } else {
      channel.createWebhook("CivilBot").queue((hook) -> {
        civilCore.getConfig().set("discord.webhook.id", hook.getId());
        civilCore.getConfig().set("discord.webhook.token", hook.getToken());
        civilCore.saveConfig();
        this.webhook = new WebhookClientBuilder(Long.parseLong(hook.getId()), hook.getToken()).build();
      });
    }
  }
}
